@@ -27,8 +27,12 @@ from sphinx.addnodes import (
 from sphinx.testing import restructuredtext
 from sphinx.testing.util import assert_node
 
+TYPE_CHECKING = False
+if TYPE_CHECKING:
+    from sphinx.application import Sphinx
 
-@pytest.mark.sphinx('html', testroot='root')
+
+@pytest.mark.sphinx('html', testroot='_blank')
 def test_pyfunction(app):
     text = (
         '.. py:function:: func1\n'
@@ -99,7 +103,7 @@ def test_pyfunction(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
+@pytest.mark.sphinx('html', testroot='_blank')
 def test_pyfunction_signature(app):
     text = '.. py:function:: hello(name: str) -> str'
     doctree = restructuredtext.parse(app, text)
@@ -146,7 +150,7 @@ def test_pyfunction_signature(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
+@pytest.mark.sphinx('html', testroot='_blank')
 def test_pyfunction_signature_full(app):
     text = (
         '.. py:function:: hello(a: str, b = 1, *args: str, '
@@ -259,7 +263,10 @@ def test_pyfunction_signature_full(app):
         doctree[1][0][1],
         [
             desc_parameterlist,
-            ([desc_parameter, nodes.inline, '*'], [desc_parameter, desc_sig_name, 'a']),
+            (
+                [desc_parameter, desc_sig_operator, nodes.abbreviation, '*'],
+                [desc_parameter, desc_sig_name, 'a'],
+            ),
         ],
     )
 
@@ -272,9 +279,9 @@ def test_pyfunction_signature_full(app):
             desc_parameterlist,
             (
                 [desc_parameter, desc_sig_name, 'a'],
-                [desc_parameter, desc_sig_operator, '/'],
+                [desc_parameter, desc_sig_operator, nodes.abbreviation, '/'],
                 [desc_parameter, desc_sig_name, 'b'],
-                [desc_parameter, desc_sig_operator, '*'],
+                [desc_parameter, desc_sig_operator, nodes.abbreviation, '*'],
                 [desc_parameter, desc_sig_name, 'c'],
             ),
         ],
@@ -289,8 +296,8 @@ def test_pyfunction_signature_full(app):
             desc_parameterlist,
             (
                 [desc_parameter, desc_sig_name, 'a'],
-                [desc_parameter, desc_sig_operator, '/'],
-                [desc_parameter, desc_sig_operator, '*'],
+                [desc_parameter, desc_sig_operator, nodes.abbreviation, '/'],
+                [desc_parameter, desc_sig_operator, nodes.abbreviation, '*'],
                 [desc_parameter, desc_sig_name, 'b'],
             ),
         ],
@@ -305,13 +312,13 @@ def test_pyfunction_signature_full(app):
             desc_parameterlist,
             (
                 [desc_parameter, desc_sig_name, 'a'],
-                [desc_parameter, desc_sig_operator, '/'],
+                [desc_parameter, desc_sig_operator, nodes.abbreviation, '/'],
             ),
         ],
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
+@pytest.mark.sphinx('html', testroot='_blank')
 def test_pyfunction_with_unary_operators(app):
     text = '.. py:function:: menu(egg=+1, bacon=-1, sausage=~1, spam=not spam)'
     doctree = restructuredtext.parse(app, text)
@@ -357,7 +364,7 @@ def test_pyfunction_with_unary_operators(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
+@pytest.mark.sphinx('html', testroot='_blank')
 def test_pyfunction_with_binary_operators(app):
     text = '.. py:function:: menu(spam=2**64)'
     doctree = restructuredtext.parse(app, text)
@@ -377,7 +384,7 @@ def test_pyfunction_with_binary_operators(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
+@pytest.mark.sphinx('html', testroot='_blank')
 def test_pyfunction_with_number_literals(app):
     text = '.. py:function:: hello(age=0x10, height=1_6_0)'
     doctree = restructuredtext.parse(app, text)
@@ -407,7 +414,7 @@ def test_pyfunction_with_number_literals(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
+@pytest.mark.sphinx('html', testroot='_blank')
 def test_pyfunction_with_union_type_operator(app):
     text = '.. py:function:: hello(age: int | None)'
     doctree = restructuredtext.parse(app, text)
@@ -437,7 +444,7 @@ def test_pyfunction_with_union_type_operator(app):
     )
 
 
-@pytest.mark.sphinx('html', testroot='root')
+@pytest.mark.sphinx('html', testroot='_blank')
 def test_optional_pyfunction_signature(app):
     text = '.. py:function:: compile(source [, filename [, symbol]]) -> ast object'
     doctree = restructuredtext.parse(app, text)
@@ -484,6 +491,58 @@ def test_optional_pyfunction_signature(app):
     )
 
 
+@pytest.mark.sphinx('html', testroot='_blank')
+def test_pyfunction_signature_with_bracket(app: Sphinx) -> None:
+    text = '.. py:function:: hello(a : ~typing.Any = <b>) -> None'
+    doctree = restructuredtext.parse(app, text)
+    assert_node(
+        doctree,
+        (
+            addnodes.index,
+            [
+                desc,
+                (
+                    [
+                        desc_signature,
+                        (
+                            [desc_name, 'hello'],
+                            desc_parameterlist,
+                            [desc_returns, pending_xref, 'None'],
+                        ),
+                    ],
+                    desc_content,
+                ),
+            ],
+        ),
+    )
+    assert_node(
+        doctree[1],
+        addnodes.desc,
+        desctype='function',
+        domain='py',
+        objtype='function',
+        no_index=False,
+    )
+    assert_node(
+        doctree[1][0][1],  # type: ignore[index]
+        (
+            [
+                desc_parameter,
+                (
+                    [desc_sig_name, 'a'],
+                    [desc_sig_punctuation, ':'],
+                    desc_sig_space,
+                    [desc_sig_name, pending_xref, 'Any'],
+                    desc_sig_space,
+                    [desc_sig_operator, '='],
+                    desc_sig_space,
+                    [nodes.inline, '<b>'],
+                ),
+            ],
+        ),
+    )
+
+
 @pytest.mark.sphinx(
     'html',
     testroot='root',
