@@ -1,5 +1,10 @@
 """Test the sphinx.environment.adapters.toctree."""
 
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
+import docutils
 import pytest
 from docutils import nodes
 from docutils.nodes import bullet_list, list_item, literal, reference, title
@@ -7,8 +12,16 @@ from docutils.nodes import bullet_list, list_item, literal, reference, title
 from sphinx import addnodes
 from sphinx.addnodes import compact_paragraph, only
 from sphinx.builders.html import StandaloneHTMLBuilder
-from sphinx.environment.adapters.toctree import document_toc, global_toctree_for_doc
+from sphinx.environment.adapters.toctree import (
+    _toctree_copy,
+    document_toc,
+    global_toctree_for_doc,
+)
 from sphinx.testing.util import assert_node
+from sphinx.util.tags import Tags
+
+if TYPE_CHECKING:
+    from sphinx.testing.util import SphinxTestApp
 
 
 @pytest.mark.sphinx('xml', testroot='toctree')
@@ -111,8 +124,19 @@ def test_process_doc(app):
         ),
     )
 
-    assert_node(toctree[1][0], [compact_paragraph, reference, 'Test for issue #1157'])
-    assert_node(toctree[1][0][0], reference, anchorname='#test-for-issue-1157')
+    assert_node(
+        toctree[1][0],
+        [
+            compact_paragraph,
+            reference,
+            'Test for combination of ‘globaltoc.html’ and hidden toctree',
+        ],
+    )
+    assert_node(
+        toctree[1][0][0],
+        reference,
+        anchorname='#test-for-combination-of-globaltoc-html-and-hidden-toctree',
+    )
     assert_node(
         toctree[1][1][0],
         addnodes.toctree,
@@ -329,7 +353,7 @@ def test_domain_objects(app):
 
 
 @pytest.mark.sphinx('dummy', testroot='toctree-domain-objects')
-def test_domain_objects_document_scoping(app):
+def test_domain_objects_document_scoping(app: SphinxTestApp) -> None:
     app.build()
 
     # tocs
@@ -450,7 +474,7 @@ def test_domain_objects_document_scoping(app):
 @pytest.mark.test_params(shared_result='test_environment_toctree_basic')
 def test_document_toc(app):
     app.build()
-    toctree = document_toc(app.env, 'index', app.builder.tags)
+    toctree = document_toc(app.env, 'index', app.tags)
 
     assert_node(
         toctree,
@@ -492,7 +516,14 @@ def test_document_toc(app):
             [bullet_list, list_item, compact_paragraph, reference, 'subsubsection'],
         ),
     )
-    assert_node(toctree[1][0], [compact_paragraph, reference, 'Test for issue #1157'])
+    assert_node(
+        toctree[1][0],
+        [
+            compact_paragraph,
+            reference,
+            'Test for combination of ‘globaltoc.html’ and hidden toctree',
+        ],
+    )
     assert_node(toctree[2][0], [compact_paragraph, reference, 'Indices and tables'])
 
 
@@ -500,8 +531,8 @@ def test_document_toc(app):
 @pytest.mark.test_params(shared_result='test_environment_toctree_basic')
 def test_document_toc_only(app):
     app.build()
-    builder = StandaloneHTMLBuilder(app, app.env)
-    toctree = document_toc(app.env, 'index', builder.tags)
+    StandaloneHTMLBuilder(app, app.env)  # adds format/builder tags
+    toctree = document_toc(app.env, 'index', app.tags)
 
     assert_node(
         toctree,
@@ -551,7 +582,14 @@ def test_document_toc_only(app):
             [bullet_list, list_item, compact_paragraph, reference, 'subsubsection'],
         ),
     )
-    assert_node(toctree[1][0], [compact_paragraph, reference, 'Test for issue #1157'])
+    assert_node(
+        toctree[1][0],
+        [
+            compact_paragraph,
+            reference,
+            'Test for combination of ‘globaltoc.html’ and hidden toctree',
+        ],
+    )
     assert_node(toctree[2][0], [compact_paragraph, reference, 'Indices and tables'])
 
 
@@ -559,7 +597,7 @@ def test_document_toc_only(app):
 @pytest.mark.test_params(shared_result='test_environment_toctree_basic')
 def test_document_toc_tocdepth(app):
     app.build()
-    toctree = document_toc(app.env, 'tocdepth', app.builder.tags)
+    toctree = document_toc(app.env, 'tocdepth', app.tags)
 
     assert_node(
         toctree,
@@ -582,7 +620,9 @@ def test_document_toc_tocdepth(app):
 @pytest.mark.test_params(shared_result='test_environment_toctree_basic')
 def test_global_toctree_for_doc(app):
     app.build()
-    toctree = global_toctree_for_doc(app.env, 'index', app.builder, collapse=False)
+    toctree = global_toctree_for_doc(
+        app.env, 'index', app.builder, tags=app.tags, collapse=False
+    )
     assert_node(
         toctree,
         [
@@ -644,7 +684,9 @@ def test_global_toctree_for_doc(app):
 @pytest.mark.test_params(shared_result='test_environment_toctree_basic')
 def test_global_toctree_for_doc_collapse(app):
     app.build()
-    toctree = global_toctree_for_doc(app.env, 'index', app.builder, collapse=True)
+    toctree = global_toctree_for_doc(
+        app.env, 'index', app.builder, tags=app.tags, collapse=True
+    )
     assert_node(
         toctree,
         [
@@ -691,7 +733,7 @@ def test_global_toctree_for_doc_collapse(app):
 def test_global_toctree_for_doc_maxdepth(app):
     app.build()
     toctree = global_toctree_for_doc(
-        app.env, 'index', app.builder, collapse=False, maxdepth=3
+        app.env, 'index', app.builder, tags=app.tags, collapse=False, maxdepth=3
     )
     assert_node(
         toctree,
@@ -782,7 +824,12 @@ def test_global_toctree_for_doc_maxdepth(app):
 def test_global_toctree_for_doc_includehidden(app):
     app.build()
     toctree = global_toctree_for_doc(
-        app.env, 'index', app.builder, collapse=False, includehidden=False
+        app.env,
+        'index',
+        app.builder,
+        tags=app.tags,
+        collapse=False,
+        includehidden=False,
     )
     assert_node(
         toctree,
@@ -875,3 +922,87 @@ def test_toctree_index(app):
         numbered=0,
         entries=[(None, 'genindex'), (None, 'modindex'), (None, 'search')],
     )
+
+
+@pytest.mark.sphinx('dummy', testroot='toctree-only')
+def test_toctree_only(app):
+    # regression test for https://github.com/sphinx-doc/sphinx/issues/13022
+    # we mainly care that this doesn't fail
+
+    if docutils.__version_info__[:2] >= (0, 22):
+        true = '1'
+    else:
+        true = 'True'
+    expected_pformat = f"""\
+<bullet_list>
+  <list_item>
+    <compact_paragraph>
+      <reference anchorname="" internal="{true}" refuri="#">
+        test-toctree-only
+    <bullet_list>
+      <list_item>
+        <compact_paragraph skip_section_number="{true}">
+          <reference anchorname="#test_toctree_only1" internal="{true}" refuri="#test_toctree_only1">
+            <literal>
+              test_toctree_only1
+      <list_item>
+        <compact_paragraph skip_section_number="{true}">
+          <reference anchorname="#test_toctree_only2" internal="{true}" refuri="#test_toctree_only2">
+            <literal>
+              test_toctree_only2
+      <list_item>
+        <compact_paragraph skip_section_number="{true}">
+          <reference anchorname="#id0" internal="{true}" refuri="#id0">
+            <literal>
+              test_toctree_only2
+"""
+    app.build()
+    toc = document_toc(app.env, 'index', app.tags)
+    assert toc.pformat('  ') == expected_pformat
+
+
+def test_toctree_copy_only():
+    # regression test for https://github.com/sphinx-doc/sphinx/issues/13022
+    # ensure ``_toctree_copy()`` properly filters out ``only`` nodes,
+    # including nested nodes.
+    node = nodes.literal('lobster!', 'lobster!')
+    node = nodes.reference('', '', node, anchorname='', internal=True, refuri='index')
+    node = addnodes.only('', node, expr='lobster')
+    node = addnodes.compact_paragraph('', '', node, skip_section_number=True)
+    node = nodes.list_item('', node)
+    node = addnodes.only('', node, expr='not spam')
+    node = addnodes.only('', node, expr='lobster')
+    node = addnodes.only('', node, expr='not ham')
+    node = nodes.bullet_list('', node)
+    # this is a tree of the shape:
+    # <bullet_list>
+    #   <only expr="not ham">
+    #     <only expr="lobster">
+    #       <only expr="not spam">
+    #         <list_item>
+    #           <compact_paragraph skip_section_number="True">
+    #             <only expr="lobster">
+    #               <reference anchorname="" internal="True" refuri="index">
+    #                 <literal>
+    #                   lobster!
+
+    tags = Tags({'lobster'})
+    toc = _toctree_copy(node, 2, 0, False, tags)
+    # the filtered ToC should look like:
+    # <bullet_list>
+    #   <list_item>
+    #     <compact_paragraph skip_section_number="True">
+    #       <reference anchorname="" internal="True" refuri="index">
+    #         <literal>
+    #           lobster!
+
+    # no only nodes should remain
+    assert list(toc.findall(addnodes.only)) == []
+
+    # the tree is preserved
+    assert isinstance(toc, nodes.bullet_list)
+    assert isinstance(toc[0], nodes.list_item)
+    assert isinstance(toc[0][0], addnodes.compact_paragraph)
+    assert isinstance(toc[0][0][0], nodes.reference)
+    assert isinstance(toc[0][0][0][0], nodes.literal)
+    assert toc[0][0][0][0][0] == nodes.Text('lobster!')
