@@ -17,6 +17,8 @@ from sphinx.testing.util import assert_node
 if TYPE_CHECKING:
     from sphinx.testing.util import SphinxTestApp
 
+FAKE_MATHJAX_URL = 'https://example.org/fake-mathjax.js'
+
 
 def has_binary(binary: str) -> bool:
     try:
@@ -124,8 +126,8 @@ def test_mathjax_options(app: SphinxTestApp) -> None:
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
     shutil.rmtree(app.outdir)
     assert (
-        '<script async="async" integrity="sha384-0123456789" '
-        'src="https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-mml-chtml.js">'
+        '<script defer="defer" integrity="sha384-0123456789" '
+        f'src="{MATHJAX_URL}">'
         '</script>'
     ) in content
 
@@ -320,10 +322,14 @@ def test_imgmath_numfig_html(app: SphinxTestApp) -> None:
 
 
 @pytest.mark.sphinx('dummy', testroot='ext-math-compat')
-def test_math_compat(app):
+def test_math_compat(app: SphinxTestApp) -> None:
     with warnings.catch_warnings(record=True):
         app.build(force_all=True)
-        doctree = app.env.get_and_resolve_doctree('index', app.builder)
+        doctree = app.env.get_and_resolve_doctree('index', app.builder, tags=app.tags)
+        doctree_0 = doctree[0]
+        assert isinstance(doctree_0, nodes.Element)
+        doctree_0_1 = doctree_0[1]
+        assert isinstance(doctree_0_1, nodes.Element)
 
         assert_node(
             doctree,
@@ -338,7 +344,7 @@ def test_math_compat(app):
             ],
         )
         assert_node(
-            doctree[0][1][1],
+            doctree_0_1[1],
             (
                 'Inline: ',
                 [nodes.math, 'E=mc^2'],
@@ -347,7 +353,7 @@ def test_math_compat(app):
             ),
         )
         assert_node(
-            doctree[0][2],
+            doctree_0[2],
             (
                 [nodes.title, 'block'],
                 [nodes.math_block, 'a^2+b^2=c^2\n\n'],
@@ -364,10 +370,10 @@ def test_math_compat(app):
     testroot='ext-math',
     confoverrides={
         'extensions': ['sphinx.ext.mathjax'],
-        'mathjax3_config': {'extensions': ['tex2jax.js']},
+        'mathjax4_config': {'extensions': ['tex2jax.js']},
     },
 )
-def test_mathjax3_config(app: SphinxTestApp) -> None:
+def test_mathjax4_config(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
@@ -376,19 +382,57 @@ def test_mathjax3_config(app: SphinxTestApp) -> None:
     assert '<script>window.MathJax = {"extensions": ["tex2jax.js"]}</script>' in content
 
 
+@pytest.mark.sphinx(
+    'html',
+    testroot='ext-math',
+    confoverrides={
+        'extensions': ['sphinx.ext.mathjax'],
+        'mathjax3_config': {'extensions': ['tex2jax.js']},
+        'mathjax_path': FAKE_MATHJAX_URL,
+    },
+)
+def test_mathjax3_config(app: SphinxTestApp) -> None:
+    app.build(force_all=True)
+
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
+    assert FAKE_MATHJAX_URL in content
+    assert f'<script defer="defer" src="{FAKE_MATHJAX_URL}">' in content
+    assert '<script>window.MathJax = {"extensions": ["tex2jax.js"]}</script>' in content
+
+
+@pytest.mark.sphinx(
+    'html',
+    testroot='ext-math',
+    confoverrides={
+        'extensions': ['sphinx.ext.mathjax'],
+        'mathjax_config_path': '_static/custom_mathjax_config.js',
+    },
+)
+def test_mathjax_config_path_config(app: SphinxTestApp) -> None:
+    app.build(force_all=True)
+
+    content = (app.outdir / 'index.html').read_text(encoding='utf8')
+    assert MATHJAX_URL in content
+    assert f'<script defer="defer" src="{MATHJAX_URL}">' in content
+    assert (
+        '<script>window.MathJax = {"extensions": ["tex2jax.js"]}\n</script>'
+    ) in content
+
+
 @pytest.mark.sphinx(
     'html',
     testroot='ext-math',
     confoverrides={
         'extensions': ['sphinx.ext.mathjax'],
         'mathjax2_config': {'extensions': ['tex2jax.js']},
+        'mathjax_path': FAKE_MATHJAX_URL,
     },
 )
 def test_mathjax2_config(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
-    assert '<script async="async" src="%s">' % MATHJAX_URL in content
+    assert f'<script async="async" src="{FAKE_MATHJAX_URL}">' in content
     assert (
         '<script type="text/x-mathjax-config">'
         'MathJax.Hub.Config({"extensions": ["tex2jax.js"]})'
@@ -402,10 +446,10 @@ def test_mathjax2_config(app: SphinxTestApp) -> None:
     confoverrides={
         'extensions': ['sphinx.ext.mathjax'],
         'mathjax_options': {'async': 'async'},
-        'mathjax3_config': {'extensions': ['tex2jax.js']},
+        'mathjax4_config': {'extensions': ['tex2jax.js']},
     },
 )
-def test_mathjax_options_async_for_mathjax3(app: SphinxTestApp) -> None:
+def test_mathjax_options_async_for_mathjax4(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
@@ -441,7 +485,7 @@ def test_mathjax_path(app: SphinxTestApp) -> None:
     app.build(force_all=True)
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
-    assert '<script async="async" src="_static/MathJax.js"></script>' in content
+    assert '<script defer="defer" src="_static/MathJax.js"></script>' in content
 
 
 @pytest.mark.sphinx(
@@ -457,7 +501,7 @@ def test_mathjax_path_config(app: SphinxTestApp) -> None:
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert (
-        '<script async="async" src="_static/MathJax.js?config=scipy-mathjax"></script>'
+        '<script defer="defer" src="_static/MathJax.js?config=scipy-mathjax"></script>'
     ) in content
 
 
@@ -486,6 +530,7 @@ def test_mathjax_is_not_installed_if_no_equations(app: SphinxTestApp) -> None:
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert 'MathJax.js' not in content
+    assert MATHJAX_URL not in content
 
 
 @pytest.mark.sphinx(
@@ -552,6 +597,7 @@ def test_mathjax_is_not_installed_if_no_equations_singlehtml(
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert 'MathJax.js' not in content
+    assert MATHJAX_URL not in content
 
 
 @pytest.mark.sphinx(
@@ -566,3 +612,19 @@ def test_mathjax_is_installed_if_included_file_has_equations_singlehtml(
 
     content = (app.outdir / 'index.html').read_text(encoding='utf8')
     assert MATHJAX_URL in content
+
+
+@pytest.mark.sphinx(
+    'html',
+    testroot='ext-math-duplicate-label',
+    confoverrides={'extensions': ['sphinx.ext.mathjax'], 'show_warning_types': True},
+)
+def test_duplicate_equation_label_warning_type(app: SphinxTestApp) -> None:
+    """Test that duplicate equation labels emit warnings with type ref.equation."""
+    app.build(force_all=True)
+
+    from sphinx._cli.util.errors import strip_escape_sequences
+
+    warnings = strip_escape_sequences(app.warning.getvalue())
+    assert 'WARNING: duplicate label of equation duplicated' in warnings
+    assert '[ref.equation]' in warnings
